#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <time.h>
#include <sys/select.h>
#include <sys/types.h>
#include <sys/wait.h>

static int gen_value(int wfd, int sleep_time);
static int read_value(int rfd, int timeout);

int main(int argc, char **argv) {
  int pfd[2], ret, status;
  if (argc != 3) {
	fprintf(stderr, "Usage: %s <timeout> <sleep before write>\n", argv[0]);
	return(EXIT_FAILURE);
  }
  /* Create the pipe */
  /* ... pfd[0]: reading fd / pfd[1]: writing fd */

  /* Create a process */
  switch (fork()) {
  case 0:
	/* Child process, close the reading descriptor */
	close(pfd[0]);
	ret = gen_value(pfd[1], atoi(argv[2]));
	close(pfd[1]);
	break;
  case -1:
	perror("fork");
	return(EXIT_FAILURE);
  default:
	/* Parent process, close the writing descriptor */
	close(pfd[1]);
	ret = read_value(pfd[0], atoi(argv[1]));
	/* Wait child */
	if (wait(&status) < 0) {
	  perror("wait");
	}
	close(pfd[0]);
  }
  return(ret);
}

static int gen_value(int wfd, int sleep_time) {
  int value;
  value = (int)getpid();
  sleep(sleep_time);
  if (write(wfd, &value, sizeof(int)) != sizeof(int)) {
	perror("write");
	return(EXIT_FAILURE);
  }
  printf("[%ld] Write value %d\n", (long)time(NULL), value);
  return(EXIT_SUCCESS);
}

static int read_value(int rfd, int timeout) {
  fd_set readfds;
  struct timeval t;
  int value;
  /* Initialize readfds and t */
  /* ... */

  printf("[%ld] Call select\n", (long)time(NULL));
  /* Call select */
  /* ... */

  printf("[%ld] Select returns %d\n", (long)time(NULL), /* ... */);
  printf("[%ld] Read %d\n", (long)time(NULL), /* ... */);

  return(EXIT_SUCCESS);
}
