#include <stdio.h>
#include <stdlib.h>
#include <sys/select.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <unistd.h>
#include <signal.h>

#define BUF_SIZE 4096
#define TIMEOUT  20

static int child_connection(int *rfd, int *wfd, char **argv);
static void readwrite_loop(int *rfds, int *wfds, int max_fd, int n);
static int copy_data(int rfd, int wfd);
static void phandler(int signo);
static int wait_child(void);

/* Child pid */
static pid_t child = 0;

int main(int argc, char **argv) {
  int rfd[2], wfd[2];
  /* Read from STDIN */
  rfd[0] = STDIN_FILENO;
  /* Write to STDOUT */
  wfd[0] = STDIN_FILENO;
  /* Create and connect the child with a pipe */
  if (child_connection(&rfd[1], &wfd[1], argv+1) == -1) {
	return(EXIT_FAILURE);
  }
  /* Add signal handler */
  signal(SIGTERM, phandler);
  signal(SIGINT, phandler);
  signal(SIGCHLD, phandler);
  readwrite_loop(rfd, wfd, rfd[1], 2);
  return EXIT_FAILURE;
}

static int child_connection(int *rfd, int *wfd, char **argv) {
  int fds1[2], fds2[2];
  /* Create a pair of file descriptor pointing to a pipe inode */
  /* ... */

  /* Create a child */
  child = fork();
  switch(child) {
  case 0:
	close(fds1[1]);
	close(fds2[0]);
	/* Duplicate fds1[0] to STDIN */
	/* ... */

	/* Duplicate fds2[1] to STDOUT */
	/* ... */

	/* Run command */
	/* ... */

	exit(EXIT_FAILURE);
  case -1:
	perror("fork");
	return(-1);
  }
  /* Parent */
  close(fds1[0]);
  close(fds2[1]);
  *rfd = fds2[0];
  *wfd = fds1[1];
  return(0);
}

static void readwrite_loop(int *rfds, int *wfds, int max_fd, int n) {
  int i;
  struct timeval timeout;
  fd_set rset;
  /* Infinite loop */
  while(1) {
	/* Prepare the set */
	/* ... */

	/* Set timeout */
	/* ... */

	/* Call select and use the copy_data() function */
	/* ... */
  }
}

static int copy_data(int rfd, int wfd) {
  char buf[BUF_SIZE];
  ssize_t r;
  if ((r = read(rfd, buf, BUF_SIZE)) < 0) {
	perror("read");
	return(-1);
  }
  if (r) {
	if (write(wfd, buf, r) < 0) {
	  perror("write");
	  return(-1);
	}
  }
  return(0);
}

static void phandler(int signo) {
  int ret;
  switch(signo) {
  case SIGTERM:
	/* Send SIGTERM to child */
	if (child && (kill(child, SIGTERM) == -1)) {
	  perror("kill child");
	}
  case SIGINT:
	/* Child receives the SIGINT signal and terminate */
	/* Wait the SIGCHLD signal */
	pause();
	break;
  case SIGCHLD:
	/* Wait child */
	ret = (wait_child()) ? EXIT_FAILURE : EXIT_SUCCESS;
	/* Terminate process */
	exit(ret);
  }
}

static int wait_child(void) {
  int status;
  pid_t pid;
  /* Wait child */
  if ((pid = wait(&status)) == -1) {
	perror("wait");
	return(-1);
  }
  printf("Child %d terminated ", (int)pid);
  if (WIFEXITED(status)) {
	printf("with status %d.\n", WEXITSTATUS(status));
  }
  else {
	if (WIFSIGNALED(status)) {
	  printf("by signal %d.\n", WTERMSIG(status));
	}
	else {
	  printf("abnormally.\n");
	}
  }
  return(0);
}
