#ifndef __MATRIX_H__
# define __MATRIX_H__

/* Matrix header */
struct matrix_hd {
  int rows; /* Rows */
  int cols; /* Columns */
};

/* Generation modes */
#define GEN_ZERO 1
#define GEN_INCR 2
#define GEN_RAND 3

/*
 * Generate a matrix file with the specified number of
 * rows and columns.
 * fd  : Matrix file descriptor
 * rows: Number of rows
 * cols: Number of columns
 * mode: Generation mode (GEN_ZERO, GEN_INCR, GEN_RAND)
 * On success, 0 is returned. On error, -1 is returned.
 */
int mtx_gen(int fd, int rows, int cols, int mode);

/*
 * Load the matrix header from the file.
 * fd : Matrix file descriptor.
 * mh : Matrix header. This is an output parameter filled by
 *      the function.
 * On success, 0 is returned. On error, -1 is returned.
 */
int mtx_get_header(int fd, struct matrix_hd *mh);

/*
 * Load element M[i][j] of the matrix.
 * fd : Matrix file descriptor.
 * mh : Matrix header.
 * i  : Row index.
 * j  : Column index.
 * value : Output paramenter M[i][j].
 * test : 1 for test mode, 0 otherwise.
 * On success, 0 is returned. On error, -1 is returned.
 */
int mtx_get_value(int fd, struct matrix_hd *mh, int i, int j, int *value, int test);

/*
 * Store the element M[i][j] of the matrix.
 * fd : Matrix file descriptor.
 * mh : Matrix header.
 * i  : Row index.
 * j  : Column index.
 * value : New value for M[i][j].
 * test : 1 for test mode, 0 otherwise.
 * On success, 0 is returned. On error, -1 is returned.
 */
int mtx_set_value(int fd, struct matrix_hd *mh, int i, int j, int value, int test);

/*
 * Load the row M[i][*] of the matrix.
 * fd : Matrix file descriptor.
 * mh : Matrix header.
 * i  : Row index.
 * values : Output paramenter M[i][*].
 * test : 1 for test mode, 0 otherwise.
 * On success, 0 is returned. On error, -1 is returned.
 */
int mtx_get_row(int fd, struct matrix_hd *mh, int i, int *values, int test);

/*
 * Store the row M[i][*] of the matrix.
 * fd : Matrix file descriptor.
 * mh : Matrix header.
 * i  : Row index.
 * values : New values for M[i][*].
 * test : 1 for test mode, 0 otherwise.
 * On success, 0 is returned. On error, -1 is returned.
 */
int mtx_set_row(int fd, struct matrix_hd *mh, int i, int *values, int test);

#endif /* !__MATRIX_H__ */
