#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include "matrix.h"

/* Options */
struct options {
  int rows;
  int cols;
  int mode;
  char *file;
};

/* Parse command-line options */
static void parse_args(int argc, char **argv, struct options *opt);

int main(int argc, char **argv) {
  int fd;
  struct options opt;
  /* Parse command-line options */
  parse_args(argc, argv, &opt);
  /* Create the matrix file */
  if ((fd = creat(opt.file, 0640)) == -1) {
	perror(opt.file);
	return(EXIT_FAILURE);
  }
  /* Generate the matrix */
  if (mtx_gen(fd, opt.rows, opt.cols, opt.mode) == -1) {
	close(fd);
	return(EXIT_FAILURE);
  }
  /* Close the file */
  if (close(fd) == -1) {
	perror("close");
	return(EXIT_FAILURE);
  }
  printf("Matrix %dx%d was successfully created to file %s.\n",
		 opt.rows, opt.cols, opt.file);
  return(EXIT_SUCCESS);
}

/* print help */
#define PRINT_HELP(prg)	\
  printf("Usage: %s -f filename -r rows -c cols [-m <z|i|r>] [-h]\n", prg)

/* print help and exit with failure status */
#define ERROR(prg,str) {\
  printf("Error: %s.\n", str);\
  PRINT_HELP(prg);\
  exit(EXIT_FAILURE);\
}

static void parse_args(int argc, char **argv, struct options *opt) {
  int c;
  /* Reset options */
  opt->file = NULL;
  opt->rows = opt->cols = opt->mode = 0;
  /* Parse command-line options */
  while ((c = getopt(argc, argv, "m:r:c:f:h")) != -1) {
	switch(c) {
	case 'm':
	  /* Mode */
	  if (opt->mode) {
		ERROR(argv[0], "You can specify only one generation mode");
	  }
	  switch(*optarg) {
	  case 'z':
		opt->mode = GEN_ZERO;
		break;
	  case 'i':
		opt->mode = GEN_INCR;
		break;
	  case 'r':
		opt->mode = GEN_RAND;
		break;
	  default:
		ERROR(argv[0], "Invalid generation mode");
	  }
	  break;
	case 'r':
	  /* Rows */
	  if (opt->rows) {
		ERROR(argv[0], "You have specified the matrix rows twice");
	  }
	  opt->rows = atoi(optarg);
	  break;
	case 'c':
	  /* Columns */
	  if (opt->cols) {
		ERROR(argv[0], "You have specified the matrix columns twice");
	  }
	  opt->cols = atoi(optarg);
	  break;
	case 'f':
	  /* Filename */
	  if (opt->file) {
		ERROR(argv[0], "You can specify only one filename");
	  }
	  opt->file = optarg;
	  break;
	case 'h':
	  PRINT_HELP(argv[0]);
	  exit(EXIT_SUCCESS);
	default:
	  ERROR(argv[0], "Invalid argument");
	}
  }
  /* Check arguments */
  if (!opt->file || (opt->rows <= 0 || opt->cols <= 0)) {
	PRINT_HELP(argv[0]);
	exit(EXIT_FAILURE);
  }
}
