#ifndef __mmap_matrix_h__
#define __mmap_matrix_h__

typedef struct matrix_hdr {
  int rows; /* Number of rows */
  int cols; /* Number of columns */
} MATRIXHDR;

typedef struct matrix {
  MATRIXHDR *hdr; /* Matrix header */
  int *data;      /* Matrix data */
} MATRIX;

#define GEN_ZERO 1
#define GEN_INCR 2
#define GEN_RAND 3

/*
 * Generate a matrix file with the specified number of
 * rows and columns.
 * fd  : Matrix file descriptor
 * rows: Number of rows
 * cols: Number of columns
 * mode: Generation mode (GEN_ZERO, GEN_INCR, GEN_RAND)
 * On success, 0 is returned. On error, -1 is returned.
 */
int matrix_gen(int fd, int rows, int cols, short mode);

/*
 * Synchronize the memory mapped matrix with the
 * matrix file and unmap the matrix from the memory.
 * m: Pointer to the matrix structure
 * On success, 0 is returned. On error, -1 is returned.
 */
int matrix_store(MATRIX *m);

/*
 * Map the matrix file into memory.
 * fd: Matrix file descriptor
 * m : Pointer to the matrix structure. This is an output
 *     parameter filled by this function
 * On success, 0 is returned. On error, -1 is returned.
 */
int matrix_load(int fd, MATRIX *m);

/*
 * Returns the matrix rows
 * m: Pointer to the MATRIX structure
 */
#define ROWS(m)  ((m)->hdr->rows)

/*
 * Returns the matrix columns
 * m: Pointer to the MATRIX structure
 */
#define COLS(m)  ((m)->hdr->cols)

/*
 * Locate the element M[i][j] of the matrix.
 * m: Pointer to the MATRIX structure
 * i: Row index. Must be in the range [0,...,rows-1]
 * j: Column index. Must be in the range [0,...,cols-1]
 */
#define M(m,i,j) (m)->data[COLS(m)*(i)+(j)]

#endif /* !__mmap_matrix_h__ */
