#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include "test.h"
#include "mmap_matrix.h"

/* See test.h */
int test1(int fd, int rep) {
  MATRIX m;
  int i, j, value;
  value = -1;
  printf("Running test1 (%d times)...\n", rep);
  while (rep--) {
	/* Load the matrix */
	if (matrix_load(fd, &m) == -1) {
	  return(-1);
	}
	/* Check all elements from the second iteration */
	if (value != -1) {
	  for (i = 0; i < ROWS(&m); i++) {
		for (j = 0; j < COLS(&m); j++) {
		  assert(M(&m, i, j) == value);
		}
	  }
	}
	/* Generate a pseudo-random integer between 0 and RAND_MAX */
	value = rand();
	/* Update all elements */
	for (i = 0; i < ROWS(&m); i++) {
	  for (j = 0; j < COLS(&m); j++) {
		M(&m, i, j) = value;
	  }
	}
	/* Store the matrix */
	if (matrix_store(&m) == -1) {
	  return(-1);
	}
  }
  printf("OK\n");
  /* Success */
  return(0);
}

/* See test.h */
int test2(int fd, int rep) {
  MATRIX m;
  int i, j, value;
  value = 0;
  printf("Running test2 (%d times)...\n", rep);
  rep = 1;
  while (rep--) {
	/* Load the matrix */
	if (matrix_load(fd, &m) == -1) {
	  return(-1);
	}
	/* Update elements  */
	for (i = 0; i < ROWS(&m); i += 2) {
	  for (j = COLS(&m)-1; j >= 0; j -= 2) {
		if (value) {
		  assert(M(&m, i, j) == (i + j + value - 1));
		}
		M(&m, i, j) = i + j + value;
	  }
	}
	for (i = 0; i < ROWS(&m); i += 2) {
	  for (j = COLS(&m)-2; j >= 0; j -= 2) {
		if (value) {
		  assert(M(&m, i, j) == (i + j + value - 1));
		}
		M(&m, i, j) = i + j + value;
	  }
	}
	for (i = 1; i < ROWS(&m); i += 2) {
	  for (j = COLS(&m)-2; j >= 0; j -= 2) {
		if (value) {
		  assert(M(&m, i, j) == (i - j + value - 1));
		}
		M(&m, i, j) = i - j + value;
	  }
	}
	for (i = 1; i < ROWS(&m); i += 2) {
	  for (j = COLS(&m)-1; j >= 0; j -= 2) {
		if (value) {
		  assert(M(&m, i, j) == (i - j + value - 1));
		}
		M(&m, i, j) = i - j + value;
	  }
	}
	/* Store the matrix */
	if (matrix_store(&m) == -1) {
	  return(-1);
	}
	/* Increment value */
	value++;
  }
  printf("OK\n");
  /* Success */
  return(0);
}
