/*
 * Questo esempio mostra l'utilizzo della select
 * con o senza timeout su un unico file descriptor (stdin).
 *
 * Uso: ./test1 [timeout]
 *
 * es: ./test1 4
 *     ./test1 0
 *     ./test1 -1
 */

#include <stdio.h>
#include <stdlib.h>
#include <sys/select.h>
#include <unistd.h>

#define DEF_TIMEOUT 5
#define SIZE 512

#define PRINT_TIME(t) {\
  if (t) {\
    printf("Timeout: %d secs and %d usecs.\n", (int)t->tv_sec, (int)t->tv_usec);\
  }\
  else {\
    printf("No timeout.\n");\
  }\
}

int main(int argc, char **argv) {
  fd_set rfds;
  struct timeval timeout, *t;
  int n;
  char buffer[SIZE];
  n = (argc == 2) ? atoi(argv[1]) : DEF_TIMEOUT;
  /* Reset set */
  FD_ZERO(&rfds);
  FD_SET(0, &rfds);
  if (n >= 0) {
	timeout.tv_sec  = n;
	timeout.tv_usec = 0;
	t = &timeout;
  }
  else {
	t = NULL;
  }
  PRINT_TIME(t);
  /* Synchronous I/O Multiplexing */
  printf("Waiting data from standard input...\n"); 
  n = select(1, &rfds, NULL, NULL, t);
  printf("select returns %d\n", n);
  PRINT_TIME(t);
  switch (n) {
  case -1:
	perror("select");
	return(1);
  case 0:
	printf("Timeout expired!\n");
	break;
  default:
	if (read(0, buffer, SIZE) < 0) {
	  perror("read");
	  return(1);
	}
	printf("Read: %s\n", buffer);
  }
  return(0);
}
