// Client program "selectClient" to demonstrate usage of FIFO
// ----------------------------------------------------------
// LSO1 AA2005-2006 - M. Verola
// ----------------------------
// It shall be used when a "selectServer" program is running.
//
// This program connects to the "selectServer" via a FIFO,
// writes a message and get a reply.
//
#include "selectCommon.h"

#define MYAPP "[selectClient]"

int main(int argc, char *argv[]) {
  int wfd, rfd, numread, numwrite, len;
  char rdbuf[MAX_BUF_SIZE];
  char fifoname[MAXFILENAME];
  char *msg, *pnum;
  
  /* Check if 2 arguments were specified. */
  if (argc != 3) {
    fprintf(stderr, "Usage : %s <index of fifo [0-N]> <string to be sent to the server>\n", argv[0]);
    return(EXIT_FAILURE);
  }

  // We should check for input arg length, to avoid buffer overflow ...
  pnum = argv[1];
  msg  = argv[2];

  // Build the name of the read-fifo of the selectServer and open it for writing.
  // The FIFO should have already been opened by the selectServer in reading,
  // otherwise this open() blocks.
  snprintf(fifoname, MAXFILENAME, "%s.%s", RFIFO_BASE_NAME, pnum);
  printf("%s Opening the FIFO \"%s\" for writing\n", MYAPP, fifoname);
  if ((wfd = open(fifoname, O_WRONLY)) == -1) {
    perror("open");
    return(EXIT_FAILURE);
  }
  
  /* Write to the pipe */
  printf("%s Writing <%s> message to the FIFO \"%s.%s\" ...\n",
		 MYAPP, msg, RFIFO_BASE_NAME, pnum);
  len = strlen(msg) + 1;  // '\0' included!
  numwrite = write(wfd, msg, len);
  close(wfd);
  printf("%s ... %d bytes written!\n", MYAPP, numwrite);
  if (numwrite != len) {
    perror("write");
    return(EXIT_FAILURE);
  }
  
  // Build the name of the write-fifo of the selectServer and open it for reading
  snprintf(fifoname, MAXFILENAME, "%s.%s", WFIFO_BASE_NAME, pnum);
  printf("%s Opening the FIFO \"%s\" for reading (might block)...\n", MYAPP, fifoname);
  if ((rfd = open(fifoname, O_RDONLY )) == -1) {
    perror("open");
    return(EXIT_FAILURE);
  }
  printf("%s ... FIFO \"%s\" opened with fd=%d\n", MYAPP, fifoname, rfd);
  
  /* Read from the pipe */
  printf("%s Reading from the FIFO \"%s.%s\" ...\n", MYAPP, WFIFO_BASE_NAME, pnum);
  numread = read(rfd, rdbuf, MAX_BUF_SIZE);
  if (numread == -1) {
    fprintf(stderr, "%s ERROR: errno=%d\n", MYAPP, errno);
    perror("read");
    return(EXIT_FAILURE);
  }
  else {
    printf("%s ... %d bytes read!\n", MYAPP, numread);  
  }

  printf("%s Message returned from server: %s\n", MYAPP, rdbuf);

  close(rfd);
  return(EXIT_SUCCESS);
}
