#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/mman.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>

static int copy_file(int fdin, int fdout);

int main(int argc, char **argv) {
  int fdin, fdout, ret;
  if (argc != 3) {
	fprintf(stderr, "Usage: %s <src> <dest>\n", argv[0]);
	return(EXIT_FAILURE);
  }
  /* Open input file */
  if ((fdin = open(argv[1], O_RDONLY)) < 0) {
	perror(argv[1]);
	return(EXIT_FAILURE);
  }
  /* Create output file */
  if ((fdout = open(argv[2], O_RDWR|O_CREAT|O_TRUNC, 0644)) < 0) {
	perror(argv[2]);
	close(fdin);
	return(EXIT_FAILURE);
  }
  /* Copy */
  ret = copy_file(fdin, fdout);
  close(fdin);
  close(fdout);
  return((ret) ? (EXIT_FAILURE) : (EXIT_SUCCESS));
}

static int copy_file(int fdin, int fdout) {
  struct stat stbuf;
  char *src, *dst;
  /* Get file status */
  if (fstat(fdin, &stbuf) == -1) {
	perror("fstat");
	return(-1);
  }
  /* Set position to new file */
  if (lseek(fdout, stbuf.st_size-1, SEEK_SET) == (off_t) -1) {
	perror("lseek");
	return(-1);
  }
  write(fdout, "", 1);
  /* Map source file into memory */
  if ((src = mmap(NULL, stbuf.st_size, PROT_READ, MAP_SHARED, fdin, 0)) == MAP_FAILED) {
	perror("src mmap");
	return(-1);
  }
  /* Map destination file into memory */
  if ((dst = mmap(NULL, stbuf.st_size, PROT_READ|PROT_WRITE, MAP_SHARED, fdout, 0)) == MAP_FAILED) {
	perror("dst mmap");
	return(-1);
  }
  /* Copy file */
  memcpy(dst, src, stbuf.st_size);
  /* Synchronize the destination file with the mapped memory */
  if (msync(dst, stbuf.st_size, MS_SYNC) == -1) {
	perror("msync");
	return(-1);
  }
  /* Unmap the files */
  if ((munmap(src, stbuf.st_size) == -1) || (munmap(dst, stbuf.st_size) == -1)) {
	perror("unmap");
	return(-1);
  }
  return(0);
}
