#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/mman.h>
#include <unistd.h>
#include <fcntl.h>

#define BASE_XOR 0x5A

static int calculate_xor(int fd);

int main(int argc, char **argv) {
  int ret, fd;
  if (argc != 2) {
	fprintf(stderr, "Usage: %s <filename>\n", argv[0]);
	return(EXIT_FAILURE);
  }
  /* Open the file */
  if ((fd = open(argv[1], O_RDWR)) == -1) {
	perror(argv[1]);
	return(EXIT_FAILURE);
  }
  /* Calculate XOR */
  ret = calculate_xor(fd);
  close(fd);
  return((ret) ? EXIT_FAILURE : EXIT_SUCCESS);
}

static int calculate_xor(int fd) {
  unsigned char *p;
  struct stat buf;
  int i;
  /* Get file status */
  if (fstat(fd, &buf) == -1) {
	perror("fstat");
	return(-1);
  }
  /* Map the file into memory */
  p = mmap(NULL, buf.st_size, PROT_READ|PROT_WRITE, MAP_SHARED, fd, 0);
  if (p == MAP_FAILED) {
	perror("mmap");
	return(-1);
  }
  /* Calculate XOR */
  p[0] ^= BASE_XOR;
  for (i = 1; i < buf.st_size; i++) {
	p[i] ^= p[i-1];
  }
  /* Synchronize the file with the memory map */
  if (msync(p, buf.st_size, MS_SYNC) == -1) {
	perror("msync");
	return(-1);
  }
  /* Unmap the file from memory */
  if (munmap(p, buf.st_size) == -1) {
	perror("munmap");
	return(-1);
  }
  return(0);
}
